<?php
/**
 * ManuAdminMod
 *
 * This is a powerful and platform independent software for administrating game servers of various kinds.
 * If you need help with installing or using this software, please visit our website at: www.ManuAdminMod.de
 * If you want to obtain additional permissions extending the license, contact us at: Webmaster@ManuAdminMod.de
 *
 * @copyright www.ManuAdminMod.de
 * @license http://www.creativecommons.org/licenses/by-nc-nd/4.0/ Creative Commons BY-NC-ND 4.0
 * @version 1.0.0-Beta+7
 **/

namespace MAM\Daemon\Engine\Frostbite3;
use MAM\Daemon\Core;
use MAM\Daemon\Libraries;
/**
 * Loop for frostbite3 engine
 *
 * @author Master of Little <masteroflittle@manuadminmod.de>
 */
class Loop {

    /**
     * Initialise rcon for Frostbite3
     */
    static function init() {
        $registry = Core\Registry::getInstance();
        $mod = $registry->mod;
        $logging = $registry->logging;
        try {
            $registry->rcon = $rcon = new Rcon\RconCommands();
            $mod->rcon = $rcon;
        } catch (\Exception $e) {
            $logging->error($e->getMessage());
        }
        $logging->notice("RCON connection established (" . $mod->getCV("connection", "ip") . ":" . $mod->getCV("connection", "port") . ")");

        $game = $rcon->rcon("version");

        if ($game[1] == "BF4") {
            $mod->setGame("bf4");
            $game = "Battlefield 4";
        } else {
            $logging->error("Unknown game detected: $game");
        }

        $logging->notice("Game detected: $game");

        try {
            $parser = new Rcon\Rcon($mod->getCV("connection", "ip"), $mod->getCV("connection", "port"));
            $parser->setRconpassword($mod->getCV("connection", "password"));
            $parser->login();
            $parser->rconCommand('admin.eventsEnabled 1');
            $registry->parser = $parser;
        } catch (\Exception $e) {
            $logging->error($e->getMessage());
        }
        $tools = new Tools();
        $registry->tools = $tools;
        $registry->rcon->tools = $tools;
        $rcon->setMaps();

        $punishment = new Libraries\Punishment\Punishment;

        $punishment->registerPunishment("warn", "warn", array("reason", "kicker"));
        $punishment->registerPunishment("kill", "kill", array("reason", "kicker"));
        $punishment->registerPunishment("ban", "ban", array("reason", "kicker"));
        $punishment->registerPunishment("kick", "kick", array("reason", "kicker"));
        $punishment->registerPunishment("tempban", "tempBan", array("reason", "kicker", "time"));

        $punishment->parseCfg();

        $registry->punishment = $punishment;
        new Libraries\Punishment\Commands();
        new Bf4\Commands();
    }

    /**
     * Main loop for Frostbite3 games
     */
    static function loop() {
        $registry = Core\Registry::getInstance();
        $mod = $registry->mod;
        $rcon = $registry->rcon;
        $logging = $registry->logging;
        $rcon->rconSay($mod->getLngString("watchingServer"));
        $mod->triggerEvent('mamStarted');
        while (true) {
            clearstatcache();
            try {
                $line = $registry->parser->receiveLogPackets();
                $action_handler = new ActionHandler();
                if ($line != false) {
                    if (!is_array($line) || empty($line[0])) {
                        continue;
                    }
                    $line["action"] = $line[0];
                    $mod->triggerEvent("logAction", $line);
                    switch ($line[0]) {
                        case "player.onKill":
                            $weapon = $line[3];
                            $body_part = ($line[4] == "true") ? "headshot" : "bodyshot";
                            $action_handler->Kill($line[1], $line[2], $weapon, $body_part);
                            break;
                        case "player.onAuthenticated":
                            $action_handler->Join($line[1]);
                            break;

                        case "player.onTeamChange":
                            if ($mod->findPlayerGuidByName($line[1]) == false) {
                                $action_handler->Join($line[1]);
                            }
                            $action_handler->JoinTeam($line[1], $line[2]);
                            break;

                        case "player.onSquadChange":
                            $action_handler->JoinSquad($line[1], $line[3]);
                            break;

                        case "player.onDisconnect":
                            /*
                              array(4) {
                              [0]=>
                              string(19) "player.onDisconnect"
                              [1]=>
                              string(7) "Master_of_Little"
                              [2]=>                               --->disconnect info
                              string(16) "PLAYER_CONN_LOST"
                              ["action"]=>
                              string(19) "player.onDisconnect"
                              }
                             */
                            $action_handler->Quit($line[1]);

                            break;

                        case "player.onChat":
                            if ($line[1] === "Server") {
                                break;
                            }
                            $action_handler->Say($line[1], $line[2]);
                            break;

                        case "server.onLevelLoaded":
                            $return = array("mapname" => $line[1], "g_gametype" => $line[2]);
                            $action_handler->NextMap($return);
                            break;

                        case "server.onRoundOver":
                            $action_handler->ResultBF4($line[1]);
                            break;

                        case "punkBuster.onMessage":
                            $action_handler->punkBuster($line[1]);
                            break;

                        default:
                            continue;
                    }
                }
                $mod->triggerEvent("everyTime");
            } catch (\Exception $exc) {
                $logging->error($exc->getMessage());
            }
        }
    }
}
