<?php
/**
 * ManuAdminMod
 *
 * This is a powerful and platform independent software for administrating game servers of various kinds.
 * If you need help with installing or using this software, please visit our website at: www.ManuAdminMod.de
 * If you want to obtain additional permissions extending the license, contact us at: Webmaster@ManuAdminMod.de
 *
 * @copyright www.ManuAdminMod.de
 * @license http://www.creativecommons.org/licenses/by-nc-nd/4.0/ Creative Commons BY-NC-ND 4.0
 * @version 1.0.0-Beta+7
 * */

namespace MAM\Daemon\Engine\Frostbite3;

use MAM\Daemon\Core\Registry;
use MAM\Daemon\Libraries\Helper\Helper;

/**
 * Class for Frostbite Rcon to manage the map list
 *
 * With this class you can manage the bf map list
 *
 * @author Master of Little <masteroflittle@manuadminmod.de>
 * @since 1.0
 * @version 1.0
 * @todo rename class/file
 * @todo move to game
 */
class Maps {
    /**
     * Rcon connection
     * @var \MAM\Daemon\Engine\Frostbite3\Rcon\Rcon
     */
    private $rcon;
    /**
     * Mod stuff
     * @todo don't want to have $mod in rcon/maps. Change
     * @var \MAM\Daemon\Core\Mod
     */
    private $mod;
    /**
     * Aviable gametypes
     * @var array
     */
    private $gamemodes;
    /**
     * Aviable maps
     * @var array
     */
    private $aviableMaps;
    /**
     * Teamnames
     * @var array
     */
    private $teamnames;
    /**
     * Squadnames
     * @var array
     */
    private $squadnames;

    /**
     * Constructor for FrostbiteMaps class
     *
     */
    public function __construct() {

        $registry = Registry::getInstance();
        $this->rcon = $registry->rcon;
        $this->logging = $registry->logging;
        $this->mod = $registry->mod;

        $this->setGamemodes();
        $this->setTeamnames();
        $this->setSquadnames();
        $this->save();
    }

    /**
     * Returns a string if object is printed
     *
     * Notice: all plugins which where moved to core needs a __tostring function
     * to get it working with the permissions system. Therefore the return of __tostring must me the
     * namespace like deamon, punishment, permissions ...
     *
     * @see http://php.net/manual/en/language.oop5.magic.php#object.tostring
     * @return string
     */
    public function __toString() {
        return "Frostbite Maps Class";
    }

    /**
     * function to set the available gamemodes
     */
    private function setGamemodes() {
        $gamemodes = Helper::parseJson(__DIR__ . DIRECTORY_SEPARATOR . $this->mod->getGame() . DIRECTORY_SEPARATOR . "data" . DIRECTORY_SEPARATOR . "gametypes.json");
        $return = array();
        $callable = array();
        $translation = array();
        foreach ($gamemodes as $short => $array) {
            $return[strtolower($short)] = strtolower($array["return"]);
            $callable[strtolower($short)] = array_map('strtolower', $array["callable"]);
            $translation[strtolower($short)] = strtolower($array["translation"]);
        }
        $this->gamemodes = array("return" => $return, "callable" => $callable, "translation" => $translation);
    }

    /**
     * Returns an array of short gametype name => long name
     * @return array
     */
    public function getGamemodesArray() {
        return $this->gamemodes;
    }

    /**
     * Returns the short gamemode of search
     * @param $search
     * @return bool|string
     */
    public function getShortGamemode($search) {
        if (array_key_exists($search, $this->gamemodes)) {
            return $search;
        } elseif ($this->getShortGamemodeFromLong($search)) {
            return $this->getShortGamemodeFromLong($search);
        } elseif ($this->getShortGamemodeFromCallable($search)) {
            return $this->getShortGamemodeFromCallable($search);
        } else {
            return false;
        }
    }

    /**
     * Returns the short gamemode name from long gamemode
     * @param $long
     * @return bool|string
     */
    public function getShortGamemodeFromLong($long) {
        $long = strtolower($long);
        foreach ($this->gamemodes["return"] as $short => $value) {
            if ($long == $value) {
                return $short;
            }
        }
        return false;
    }

    /**
     * Returns the short gamemode name from callable
     * @param $callable
     * @return bool|string
     */
    public function getShortGamemodeFromCallable($callable) {
        foreach ($this->gamemodes["callable"] as $short => $array) {
            if (in_array($callable, $array)) {
                return $short;
            }
        }
        return false;
    }

    /**
     * function to set the teamnames
     */
    private function setTeamnames() {
        $this->teamnames = Helper::parseJson(__DIR__ . DIRECTORY_SEPARATOR . $this->mod->getGame() . DIRECTORY_SEPARATOR . "data" . DIRECTORY_SEPARATOR . "teams.json")["teamnames"];
    }

    /**
     * Returns an array of squadindex => squadname
     * @return array
     */
    public function getTeamnames() {
        return $this->teamnames;
    }

    /**
     * function to set the squadnames
     */
    private function setSquadnames() {
        $this->squadnames = Helper::parseJson(__DIR__ . DIRECTORY_SEPARATOR . $this->mod->getGame() . DIRECTORY_SEPARATOR . "data" . DIRECTORY_SEPARATOR . "teams.json")["squadnames"];
    }

    /**
     * Returns an array of teamnameindex => teamname
     * @return array
     */
    public function getSquadnames() {
        return $this->squadnames;
    }

    /**
     * Returns MapID from currrent map
     * @return int
     */
    public function getCurrentMapID() {
        return $this->rcon->rcon("mapList.getMapIndices")[1];
    }

    /**
     * Returns MapID from next map
     * @return int
     */
    public function getNextMapID() {
        return $this->rcon->rcon("mapList.getMapIndices")[2];
    }

    /**
     * Returns mapname of index
     * @param int $index
     * @return string Mapname
     */
    public function getMapName($index) {
        return $this->rcon->rcon("mapList.list $index")[3];
    }

    /**
     * Returns gametype of index
     * @param int $index
     * @return string gametype
     */
    public function getGametype($index) {
        return $this->rcon->rcon("mapList.list $index")[4];
    }

    /**
     * Return round now played
     * @return int
     */
    public function getRound() {
        return $this->rcon->rcon("mapList.getRounds")[1];
    }

    /**
     * Return max rounds
     * @return int
     */
    public function getMaxRounds() {
        return $this->rcon->rcon("mapList.getRounds")[2];
    }

    /**
     * Returns maps aviable for gametype or gametype aviable for maps
     */
    public function aviableMaps() {
        $out = $this->rcon->rcon("mapList.availableMaps perGameMode");
        $this->aviableMaps = array();
        unset($out[0]);
        $key = "none";
        foreach ($out as $value) {
            if (array_key_exists(strtolower($value), $this->gamemodes["callable"])) {
                $key = strtolower($value);
            } else {
                $this->aviableMaps[$key][] = strtolower($value);
            }
        }
    }

    /**
     * Adds a map with gametype and rounds at offset to the maplist
     * if offset is false map will be added at the end of the maplist
     * @param string $map
     * @param string $gamemode
     * @param int $rounds
     * @param int $offset
     * @return boolean
     */
    public function add($map, $gamemode, $rounds, $offset = 1) {
        $this->aviableMaps();
        $gamemode = strtolower($gamemode);
        $map = strtolower($map);
        if (array_key_exists($gamemode, $this->aviableMaps) && in_array($map, $this->aviableMaps[$gamemode])) {
            $gamemode = strtoupper($gamemode);
            $this->save();
            return $this->rcon->getStatus($this->rcon->rcon("mapList.add $map $gamemode $rounds $offset"));
        } else {
            return false;
        }
    }

    /**
     * Cleans the maplist
     * @return boolean
     */
    public function clear() {
        return $this->rcon->getStatus($this->rcon->rcon("mapList.clear"));
    }

    /**
     * Rote to next map
     * @return boolean
     */
    public function mapRotate() {
        $this->setNextMapIndex($this->getNextMapID());
        return $this->roundRotate();
    }

    /**
     * Rotate to next round
     * @return boolean
     */
    public function roundRotate() {
        return $this->rcon->getStatus($this->rcon->rcon("mapList.runNextRound"));
    }

    /**
     * Restart current round
     * @return boolean
     */
    public function restartRound() {
        return $this->rcon->getStatus($this->rcon->rcon("mapList.restartRound"));
    }

    /**
     * set next map with index from maplist
     * @param int $index
     * @return boolean
     */
    public function setNextMapIndex($index) {
        return $this->rcon->getStatus($this->rcon->rcon("mapList.setNextMapIndex $index"));
    }

    /**
     * Ends round with winner team with ID
     * @param int $winner
     * @return boolean
     */
    public function endRound($winner = 0) {
        return $this->rcon->getStatus($this->rcon->rcon("mapList.endRound $winner"));
    }

    /**
     * Removes map with index from maplist
     * @param int $index
     * @return boolean
     */
    public function remove($index) {
        return $this->rcon->getStatus($this->rcon->rcon("mapList.remove $index"));
    }

    /**
     * Saves maplist to HDD
     * @return boolean
     */
    public function save() {
        return $this->rcon->getStatus($this->rcon->rcon("mapList.save"));
    }

    /**
     * Loads maplist from HDD
     * @return boolean
     */
    public function load() {
        return $this->rcon->getStatus($this->rcon->rcon("mapList.load"));
    }
}
