<?php
/**
 * ManuAdminMod
 *
 * This is a powerful and platform independent software for administrating game servers of various kinds.
 * If you need help with installing or using this software, please visit our website at: www.ManuAdminMod.de
 * If you want to obtain additional permissions extending the license, contact us at: Webmaster@ManuAdminMod.de
 *
 * @copyright www.ManuAdminMod.de
 * @license http://www.creativecommons.org/licenses/by-nc-nd/4.0/ Creative Commons BY-NC-ND 4.0
 * @version 1.0.0-Beta+7
 **/

namespace MAM\Daemon\Engine\Frostbite3;

/**
 * Class to handle the frostbite rcon easier
 *
 *
 * @author Master of Little <masteroflittle@manuadminmod.de>
 */
class Tools {
    /**
     * Mod
     *
     * @var \MAM\Daemon\Core\Mod
     */
    private $mod;
    /**
     * Rcon
     *
     * @var \MAM\Daemon\Engine\Frostbite3\Rcon\Rcon
     */
    private $rcon;
    /**
     * RconLog
     *
     * @var \MAM\Daemon\Engine\Frostbite3\Rcon\RconLog
     */
    private $rconLog;
    /**
     * Players
     *
     * @var \MAM\Daemon\Engine\Frostbite3\Player
     */
    private $players;
    /**
     * Logging
     *
     * @var \MAM\Daemon\Libraries\Logging\LogHandler
     */
    private $logging;
    private $pb_name;

    /**
     * Constructor for Frostbite3/Tools
     *
     */
    public function __construct() {
        $registry = \MAM\Daemon\Core\Registry::getInstance();
        $this->mod = $registry->mod;
        $this->players = & $registry->players;
        $this->rcon = $registry->rcon;
        $this->logging = $registry->logging;
        $this->rconLog = $registry->parser; // -------rconLog aus daemon --------


        $this->checkPbEnabled();
        $this->getPbPrefix();
    }

    /**
     * Checks if Punkbuster is activated
     */
    private function checkPbEnabled() {

        if ($this->rcon->rcon('punkBuster.isActive')[1] === "false") {
            $this->logging->error("Punkbuster is disabled. Can't use ManuAdminMod");
        }
        $this->logging->notice("Punkbuster is enabled. Finish initialisation");
        $this->rcon->rcon(array('punkBuster.pb_sv_command', 'pb_sv_autoupdban 1'));
    }

    private function getPbPrefix() {
        do {
            sleep(1);
            $this->rcon->rcon(array('punkBuster.pb_sv_command', 'pb_sv_ver'));
            $out = $this->rconLog->receiveLogPackets();
        } while (!preg_match("/PunkBuster Server for/", $out[1]));
        $this->pb_name = explode(":", $out[1])[0];
        $this->logging->notice("Punkbuster prefix: " . $this->pb_name);
    }

    /**
     * Returns a string if object is printed
     *
     * Notice: all plugins which where moved to core needs a __toString function
     * to get it working with the permissions system. Therefore the return of __toString must be the
     * namespace like deamon, punishment, permissions ...
     *
     * @see http://php.net/manual/en/language.oop5.magic.php#object.tostring
     * @return string
     */
    public function __toString() {
        return "Frostbite Tools Class";
    }

    /**
     * Returns a punkBuster Status array
     * @return array
     */
    public function punkBusterStatus() {
        /*
          array(9) {
          [0]=>
          string(1) "1"
          [1]=>
          string(32) "a3a393adc4fa29fd7e7b393d5f6c9c36"
          [2]=>
          string(18) "84.149.147.28:3659"
          [3]=>
          string(2) "OK"
          [4]=>
          string(1) "1"
          [5]=>
          string(3) "3.0"
          [6]=>
          string(1) "0"
          [7]=>
          string(3) "(V)"
          [8]=>
          string(16) "Master_of_Little"
          }
         */
        $pb_log = array();
        $start = false;
        $this->logging->notice("Get Punkbuster player list");
        do {
            if (!$start) {
                $this->rcon->rcon('punkBuster.pb_sv_command pb_sv_plist');
            }
            $out = $this->rconLog->receiveLogPackets()[1];
            if ($start == true) {
                $pb_log[] = $out;
             //   var_dump($out);
            }
            if (preg_match("/" . $this->pb_name . ": Player List:/", $out)) {
                $start = true;
            }
        } while ($out === false || !preg_match("/" . $this->pb_name . ": End of Player List/", $out));
        $pb_list = array();
        $pattern = array("/" . $this->pb_name . ": End of Player List \([0-9]* Player[s]*\)/", "/" . $this->pb_name . ": /", "/\"/", "/\(-\)/", "/\([\s]\)/", "/[\s]+/");
        $replacement = array("", "", "", "", "(-)", " ");
        // var_dump($out);
        foreach ($pb_log as $value) {
            $list = preg_replace($pattern, $replacement, $value);
            if ($list === "") {
                continue;
            }
            $array = explode(" ", $list);
            //var_dump($array);
            $return[0] = $array[0]; //PID
            $return[1] = $array[1]; //PBGUID
            $return[2] = $array[2]; //IP:PORT
            $return[3] = $array[8]; //NAME


            $pb_list[$return[3]] = $return;
        }
        return $pb_list;
    }

    /**
     * Handles the Punkbuster Log if a player joins
     * @param string $out Logline
     * @param bool $newconnection
     * @return array
     */
    public function punkBusterPlayerJoin($out, $newconnection = false) {

        //  $cmd = $this->rcon->rcon('punkBuster.pb_sv_command pb_sv_plist');
        //  $out = $this->rconLog->rconLog();
        // var_dump($out);
        //$pblist = array();
        if ($newconnection) {
            // $pattern = array("/" . $this->pb_name . ": Player GUID Computed /", "/\(-\)/", "/[\s]+/", "/\(slot #/", "/\)/");
            $pattern = array("/" . $this->pb_name . ": New Connection /", "/\(-\)/", "/[\s]+/", "/\(slot #/", "/\)/", "/\"/"); // "PunkBuster Server: New Connection (slot #1) 84.149.190.26:3659 [?] "Master_of_Little" (seq 11412273)"
            $replacement = array("", "", " ", "", "", "");
            $list = preg_replace($pattern, $replacement, $out);
            //var_dump($list);
            $array = explode(" ", $list);
            //var_dump($array);
            $return[0] = $array[0]; //PID
            $return[1] = false; //PBGUID
            $return[2] = $array[1]; //IP:PORT
            $return[3] = $array[3]; //NAME
        } else {
            $pattern = array("/" . $this->pb_name . ": Player GUID Computed /", "/\(-\)/", "/[\s]+/", "/\(slot #/", "/\)/");
            $replacement = array("", "", " ", "", "");
            $list = preg_replace($pattern, $replacement, $out);
            $array = explode(" ", $list);
            $return[0] = $array[1]; //PID
            $return[1] = $array[0]; //PBGUID
            $return[2] = $array[2]; //IP:PORT
            $return[3] = $array[3]; //NAME
        }
        //$pblist[$return[3]] = $return;
        return $return;
    }

    /**
     * Handles the Punkbuster Log if a player joins
     * @param $name
     * @param $line
     */
    public function punkBusterPlayer($name, $line) {
        if (preg_match("/" . $this->pb_name . ": Player GUID Computed/", $line)) {
            $this->renewPlayer($name, $this->punkBusterPlayerJoin($line, false));
        } elseif (preg_match("/" . $this->pb_name . ": New Connection/", $line)) {
            $this->renewPlayer($name, $this->punkBusterPlayerJoin($line, true));
        } else {
            return;
        }
    }

    /**
     * Updates a Player: Updates PID,IP
     *
     * @param string $name
     * @param array $pb
     */
    public function renewPlayer($name = '', $pb = array()) {
        if ($name === '' || $name === false) {
            $guid = $this->mod->findPlayerGuidByName($pb[3]);
            if ($guid === false) {
                $this->logging->debug('tools/renewPlayer() failed with $pb: ' . implode(', ', $pb));
                return;
            }
            $ipPort = explode(":", $pb[2]);
            $this->players[$guid]->setIP($ipPort[0]);
            $this->players[$guid]->update($pb[0]);
            if ($pb[1] !== false) {
                $this->players[$guid]->setPBGuid($pb[1]);
                $this->logging->info("PB-Update: $pb[3], IP: $ipPort[0], PID: $pb[0], PBGuid: $pb[1]");
            } else {
                $this->logging->info("PB-Update: $pb[3], IP: $ipPort[0], PID: $pb[0]");
                $this->mod->triggerEvent('playerIPUpdated', $guid);
            }
            $this->mod->triggerEvent('playerPbUpdated', $guid);
        } else {
            $guid = $this->mod->findPlayerGuidByName($name);
            $pb = $this->punkBusterStatus();
            $ipPort = explode(":", $pb[$name][2]);
            $this->players[$guid]->setIP($ipPort[0]);
        }
//        $this->players[$this->mod->findPlayerGuidByName($name)]->update($pb[$name][0]);

    }

}
