<?php
/**
 * ManuAdminMod
 *
 * This is a powerful and platform independent software for administrating game servers of various kinds.
 * If you need help with installing or using this software, please visit our website at: www.ManuAdminMod.de
 * If you want to obtain additional permissions extending the license, contact us at: Webmaster@ManuAdminMod.de
 *
 * @copyright www.ManuAdminMod.de
 * @license http://www.creativecommons.org/licenses/by-nc-nd/4.0/ Creative Commons BY-NC-ND 4.0
 * @version 1.0.0-Beta+7
 * */

namespace MAM\Daemon\Engine\Quake3;

use MAM\Daemon\Core\Registry;

/**
 * Player Class
 *
 * This class contains all player relevant functions/options
 *
 * @author Mirko911 <mirko911@manuadminmod.de>
 * @todo rework docblock -> english
 */
class Player extends \MAM\Daemon\Core\Player {
    /**
     * How much damage was given
     *
     * @var int
     */
    public $damagegiven = 0;
    /**
     * How much team damage was given
     *
     * @var int
     */
    public $teamdamagegiven = 0;
    /**
     * How much damage was taken
     *
     * @var int
     */
    public $damagetaken = 0;
    /**
     * How much vehicle damage was given
     *
     * @var int
     */
    public $vehicledamagegiven = 0;
    /**
     * How much team vehicle damage was given
     *
     * @var int
     */
    public $teamvehicledamagegiven = 0;
    /**
     * How much damage the actor has given his enemy.
     *
     * @var int
     */
    public $actorsdamagegiven = 0;
    /**
     * How much damage the actor has given his teammates.
     *
     * @var int
     */
    public $teamactorsdamagegiven = 0;
    /**
     * Kill Counter
     *
     * @var int
     */
    public $kills = 0;
    /**
     * Death Counter
     *
     * @var int
     */
    public $deaths = 0;
    /**
     * Teamkill Counter
     * @var int
     */
    public $teamkills = 0;
    /**
     * Teamdeath counter
     *
     * @var int
     */
    public $teamdeaths = 0;
    /**
     * Selfkill Counter
     *
     * @var int
     */
    public $selfkills = 0;
    /**
     * Headshots Counter
     *
     * @var int
     */
    public $headshots = 0;
    /**
     * Was the last death a teamkill?
     * @var boolean
     */
    public $lastdeathwastk = false;
    /**
     * Was the last kill a teamkill?
     *
     * @var boolean
     */
    public $lastkillwastk = false;
    /**
     * Who was the last killer?
     * @var string guid
     */
    protected $lastkiller = false;
    /**
     * Who was the last victim?
     * @var string guid
     */
    protected $lastvictim = false;

    /**
     * Construct
     *
     * @param String $guid Guid of player
     * @param int $pid PID of Player
     * @param string $name  Name of Player
     */
    public function __construct($guid, $pid, $name) {
        parent::__construct($guid, $pid, $name);
    }

    /**
     * Returns the last Victim of this Player [Victim = Opfer]
     * @return bool|string false or last Victim
     */
    public function getLastVictim() {
        if (!$this->lastvictim || !array_key_exists($this->lastvictim, $this->players)) {
            return false;
        }
        return $this->lastvictim;
    }

    /**
     * Returns the last Killer of this Palyer
     * @return boolean
     */
    public function getLastKiller() {
        if (!$this->lastkiller || !array_key_exists($this->lastkiller, $this->players)) {
            return false;
        }
        return $this->lastkiller;
    }

    /**
     * Resets the Team
     */
    public function resetTeam() {
        $this->team = "none";
    }

    /**
     * Set this Player to a team
     * @param string $team Teamname
     */
    public function setTeam($team) {
        if ($team != $this->team && !empty($team)) {
            $oldteam = $this->team;
            $this->team = $team;

            $this->mod->triggerEvent("playerTeamChange", array($this->guid, $oldteam));
        }
    }

    /**
     * Returns the Team of a Player
     * @return String Name of the team
     */
    public function getTeam() {
        return $this->team;
    }

    /**
     * checks if the player on the server
     * @return string $info ?????
     */
    public function dumpUser() {
        $return = $this->rcon->rcon("dumpuser \"$this->name\"");
        if (preg_match('|^Player .+ is not on the server\s?$|', $return)) {
            return false;
        }
        $return = explode("\n", $return);
        $info = array();
        foreach ($return as $value) {
            if (preg_match('|^([a-z0-9_]+)\s+(.*)$|i', $value, $subpatterns)) {
                $info[$subpatterns[1]] = $subpatterns[2];
            }
        }
        return $info;
    }

    /**
     * Updates the Players PID, Name if it was changed and loggs it
     * @param int $pid new Player-ID
     * @param bool|string $name new Player-Name
     */
    public function update($pid, $name = false) {
        $name = trim($name);
        if ($name != false && $name != $this->name) {
            $this->logging->info("Player '$this->name' changed name to '$name', PID: $this->pid, GUID: $this->guid");
            $oldname = $this->name;
            $this->name = $name;

            $nick = $this->database->row('nicks', array('user' => $this->getUserId(), 'nick' => $name));
            if (empty($nick)) {
                $this->database->insert('nicks', array('nick' => $name, 'user' => $this->getUserId()));
            }
            $this->mod->triggerEvent("playerNameChange", array($this->guid, $oldname, $name));
        }
        if ($pid != $this->pid) {
            $this->logging->info("Player '$this->name's PID was changed from $this->pid to $pid, GUID: $this->guid");
            $oldpid = $this->pid;
            $this->pid = $pid;
            $this->mod->triggerEvent("playerPIDChange", array($this->guid, $oldpid, $pid));
        }
    }

    /**
     * Returns the IP of a Player
     *
     * If ip can't get after 5 trys mod will use no ip
     * This function also sets the ip for all other players, so you
     * need less rcon requests
     *
     * @return String IP of the Player
     */
    public function getIP() {
        //Check if ip is empty
        $registy = \MAM\Daemon\Core\Registry::getInstance();

        if (empty($this->ip)) {
            $max_trys = (!$registy->iceops) ? 5 : 10;
            $i = 0;
            while ($i < $max_trys) {
                $status = $this->rcon->rconPlayerList(); //Search Playerlist
                foreach ($status as $line) { //for every player
                    if (array_key_exists($line['guid'], $this->players)) {
                        $this->players[$line["guid"]]->setIP(trim($line['ip']));
                    } elseif ($line['guid'] == $this->getGuid()) {
                        $this->setIP(trim($line['ip']));
                        $i = $max_trys;
                        break;
                    }
                }
                sleep(0.2); //Wait some time
                $i++;
            }
        }
        //Return real IP or return empty string
        return $this->ip;
    }

    /**
     * Sets the ip of a player
     *
     * This function is used to set the ip of a player in
     * the getIP() function.
     *
     * This function only sets the ip if ip is undefined
     * or not valid. You also need to pass a valid IPv4 or IPv6 address
     * to this function
     *
     * @param string $ip Unchecked IP
     * @return string IP
     */
    public function setIP($ip) {
        if ((empty($this->ip) || !inet_pton($this->ip)) && inet_pton($ip) != false) {
            $this->ip = $ip;
            return $ip;
        }
    }

    /**
     * Realisiert einen Kick mittels PB. Ist dies nicht möglich so wird über RCON gekickt
     * @param string|bool $reason
     * @param string|bool $kicker kickers guid
     * @return boolean
     */
    public function kick($reason = false, $kicker = false) {

        if ($this->permissions->hasPermissions($this->getGuid(), 'punishment.ignore.kick')) {
            if ($kicker != false) {
                $this->players[$kicker]->say($this->mod->getLngString("ignoreKick"));
            }
            return false;
        }
        //Check if there should be an error, if no reason was given
        if (!$reason && $this->mod->getCV("punishment", "forcereason") && $kicker != false) {
            $this->players[$kicker]->say($this->mod->getLngString("forceReason"));
            return false;
        } elseif (!$reason) {
            $reason = $this->mod->getCV("punishment", "defkickreason");
        }

        if ($this->mod->getCV("punishment", "punkbuster")) {
            $result = $this->rcon->rcon("pb_sv_kick " . $this->getPbid() . " 0 $reason^7");
            // ^3PunkBuster Server: Matched: ^1FN^0! ^2manu (slot #1)
            // 0:^1FN^0! ^2manu PunkBuster kicked player '^1FN^0! ^2manu' (for 0 minutes) ... Kicked by Admin [Admin Decision]
            // ^3PunkBuster Server: Kick Command Issued (Kicked by Admin [Admin Decision]) for (slot#1) 192.168.2.21:28960 c90dc162f630049cec0cb632e5e0aa50 ^1FN^0! ^2manu
            if (stripos($result, "Kick Command Issued") === false) {
                $this->logging->warning("Error kicking player '" . $this->name . "' with pb_sv_kick, using clientkick, PID: " . $this->pid . ", GUID:" . $this->guid);
                $this->rcon->rcon("clientkick " . $this->pid);
            }
        } else {
            for ($i = 0; $i < 5; $i++) {
                $this->say($this->mod->getLngString("playerKickReasonNoPb", array("<REASON>"), array($reason)));
            }
            if (!\MAM\Daemon\Core\Registry::getInstance()->iceops) {
                \MAM\Daemon\Core\Registry::getInstance()->jobs->addSingleJob(3, array($this->rcon, "rcon"), array("clientkick " . $this->pid));
            } else {
                $this->rcon->rcon("clientkick " . $this->pid . " " . $reason);
            }
        }

        $kickerlog = ($kicker !== false) ? $this->players[$kicker]->getName() : "MOD";
        $this->logging->info("Player '$this->name' got kicked (by: $kickerlog), reason: $reason, PID: $this->pid, GUID: $this->guid");


        if ($this->mod->getCV("punishment", "announce")) {
            if ($kicker !== false) {
                $search = array("<PLAYER>", "<KICKER>", "<REASON>");
                $replace = array($this->getName(), $this->players[$kicker]->getName(), $reason ? $reason : $this->mod->getCV("punishment", "defkickreason"));
                $this->rcon->rconSay($this->mod->getLngString("playerKickedPublicMsg", $search, $replace));
            } else {
                $search = array("<PLAYER>", "<REASON>");
                $replace = array($this->getName(), $reason ? $reason : $this->mod->getCV("punishment", "defkickreason"));
                $this->rcon->rconSay($this->mod->getLngString("playerKickedPublicMsgAuto", $search, $replace));
            }
        }
        //Create timed event to avoid mam lags due sleep
        $this->mod->triggerEvent("playerKicked", array($this->guid, $reason, $kicker));

        return true;
    }

    /**
     * Realisiert einen Bann mittels PB. Ist dies nicht möglich so wird über RCON gebannt
     * @param string|bool $reason
     * @param string|bool $kicker kickers guid
     * @return boolean
     */
    public function ban($reason = false, $kicker = false) {
        if ($this->permissions->hasPermissions($this->getGuid(), 'punishment.ignore.ban')) {
            if ($kicker != false) {
                $this->players[$kicker]->say($this->mod->getLngString("ignoreBan"));
            }
            return false;
        }
        //Check if there should be an error, if no reason was given
        if (!$reason && $this->mod->getCV("punishment", "forcereason") && $kicker != false) {
            $this->players[$kicker]->say($this->mod->getLngString("forceReason"));
            return false;
        } elseif (!$reason) {
            $reason = $this->mod->getCV("punishment", "defkickreason");
        }
        if ($this->mod->getCV("punishment", "punkbuster")) {
            /* $dump = $this->dumpUser();
              if (!$dump) {
              $this->logging-warning("Couldn't dump user '".$this->name."', can't ban him with pb_sv_ban, using banclient, PID: ".$this->pid.", GUID:".$this->guid);
              $this->rcon->rcon("banClient " . $this->pid);
              } */
            $type = "pb";
            $result = $this->rcon->rcon("pb_sv_ban " . $this->getPbid() . " $reason^7");
            if (stripos($result, "Kick/Ban Command Issued") === false) {
                $this->logging->warning("Error kicking player '" . $this->name . "' with pb_sv_kick, using clientkick, PID: " . $this->pid . ", GUID:" . $this->guid);
                $this->rcon->rcon("banClient " . $this->pid);
                $type = "guid";
            }
        } else {
            $type = "guid";
            for ($i = 0; $i < 5; $i++) {
                $this->say($this->mod->getLngString("playerBanReasonNoPb", array("<REASON>"), array($reason)));
            }
            if (!\MAM\Daemon\Core\Registry::getInstance()->iceops) {
                \MAM\Daemon\Core\Registry::getInstance()->jobs->addSingleJob(3, array($this->rcon, "rcon"), array("banClient " . $this->pid));
            } else {
                $this->rcon->rcon("banUser " . $this->pid . " " . $reason);
            }
        }

        $kickerlog = ($kicker !== false) ? $this->players[$kicker]->getName() : "MOD";
        $this->logging->info("Player '$this->name' got banned (by: $kickerlog), reason: $reason, PID: $this->pid, GUID: $this->guid");

        $banhelper = Registry::getInstance()->banhelper;
        $banhelper->writeBan($this->guid, substr($this->guid, -8), $this->name, $reason, 0, $type);

        if ($this->mod->getCV("punishment", "announce")) {
            if ($kicker !== false) {
                $search = array("<PLAYER>", "<KICKER>", "<REASON>");
                $replace = array($this->getName(), $this->players[$kicker]->getName(), $reason ? $reason : $this->mod->getCV("punishment", "defbanreason"));
                $this->rcon->rconSay($this->mod->getLngString("playerBannedPublicMsg", $search, $replace));
            } else {
                $search = array("<PLAYER>", "<REASON>");
                $replace = array($this->getName(), $reason ? $reason : $this->mod->getCV("punishment", "defbanreason"));
                $this->rcon->rconSay($this->mod->getLngString("playerBannedPublicMsgAuto", $search, $replace));
            }
        }

        $this->mod->triggerEvent("playerBanned", array($this->guid, $reason, $kicker));

        return true;
    }

    /**
     * Realisiert einen TempBann mittels PB. Ist dies nicht möglich so wird über RCON gebannt
     * @param string|bool $reason
     * @param string|bool $kicker kickers guid
     * @param bool|int $time ban time in seconds
     * @return boolean
     */
    public function tempBan($reason = false, $kicker = false, $time = false) {
        if ($this->permissions->hasPermissions($this->getGuid(), 'punishment.ignore.tempban')) {
            if ($kicker != false) {
                $this->players[$kicker]->say($this->mod->getLngString("ignoreTempBan"));
            }
            return false;
        }
        //Check if there should be an error, if no reason was given
        if (!$reason && $this->mod->getCV("punishment", "forcereason") && $kicker != false) {
            $this->players[$kicker]->say($this->mod->getLngString("forceReason"));
            return false;
        } elseif (!$reason) {
            $reason = $this->mod->getCV("punishment", "defkickreason");
        }
        if ($this->mod->getCV("punishment", "punkbuster")) {
            if (!$time || !is_numeric($time) || $time < 0) {
                $time = $this->mod->getCV("punishment", "deftempbandur");
            } else {
                $time = (int) $time;
            }
            $type = "pb";
            $result = $this->rcon->rcon("pb_sv_kick " . $this->getPbid() . " $time $reason^7");
            if (stripos($result, "Kick Command Issued") === false) {
                $this->logging->warning("Error kicking player '" . $this->name . "' with pb_sv_kick, using clientkick, PID: " . $this->pid . ", GUID:" . $this->guid);
                $this->rcon->rcon("tempBanClient " . $this->pid);
                $type = "guid";
            }
        } else {
            $type = "guid";
            for ($i = 0; $i < 5; $i++) {
                $this->say($this->mod->getLngString("playerBanReasonNoPb", array("<REASON>"), array($reason)));
            }
            if (!\MAM\Daemon\Core\Registry::getInstance()->iceops) {
                \MAM\Daemon\Core\Registry::getInstance()->jobs->addSingleJob(3, array($this->rcon, "rcon"), array("tempBanClient " . $this->pid));
            } else {
                $this->rcon->rcon("tempban " . $this->pid . " " . $time . "m " . $reason);
            }
        }

        $kickerlog = ($kicker !== false) ? $this->players[$kicker]->getName() : "MOD";
        $this->logging->info("Player '$this->name' got temporarily banned for $time minutes (by: $kickerlog), reason: $reason, PID: $this->pid, GUID: $this->guid");

        $banhelper = Registry::getInstance()->banhelper;
        $banhelper->writeBan($this->guid, substr($this->guid, -8), $this->name, $reason, time() + $time * 60, $type);

        if ($this->mod->getCV("punishment", "announce")) {
            if ($kicker !== false) {
                $search = array("<PLAYER>", "<KICKER>", "<REASON>", "<TIME>");
                $replace = array($this->getName(), $this->players[$kicker]->getName(), $reason ? $reason : $this->mod->getCV("punishment", "defbanreason"), ($time) ? $time : $this->mod->getCV("punishment", "deftempbandur"));
                $this->rcon->rconSay($this->mod->getLngString("playerTempBannedPublicMsg", $search, $replace));
            } else {
                $search = array("<PLAYER>", "<REASON>", "<TIME>");
                $replace = array($this->getName(), $reason ? $reason : $this->mod->getCV("punishment", "defbanreason"), ($time) ? $time : $this->mod->getCV("punishment", "deftempbandur"));
                $this->rcon->rconSay($this->mod->getLngString("playerTempBannedPublicMsgAuto", $search, $replace));
            }
        }

        $this->mod->triggerEvent("playerTempBanned", array($this->guid, $reason, $time ? $time : $this->mod->getCV("punishment", "deftempbandur"), $kicker));

        return true;
    }

    /**
     * Notice that the player dies and start events cause of his death
     *
     * @param string $type deathtype like teamkill selfkill
     * @param string $killer the killer who killed the player
     * @param string $weapon the weapon name which the player was killed
     * @param int $damage the damage dealed to the player
     * @param string $bodypart the body part
     */
    public function death($type, $killer, $weapon, $damage, $bodypart) {
        $this->lastkiller = $killer;
        $this->lastweapondeath = $weapon;
        $this->deaths ++;
        $this->damagetaken += $damage;
        $this->lastdeathwastk = false;

        if ($type == "teamkill") {
            $this->teamdeaths ++;
            $this->lastdeathwastk = true;
            $this->mod->triggerEvent("playerTeamDeath", array($this->guid, $killer, $weapon, $damage, $bodypart));
        } elseif ($type == "selfkill") {
            $this->selfkills ++;
            $this->lastkiller = $this->guid;
            $this->lastvictim = $this->guid;
            $this->lastweaponkill = $weapon;
            $this->lastkillwastk = false;
            $this->mod->triggerEvent("playerSelfKill", array($this->guid, $weapon, $damage, $bodypart));
        } else {
            $this->mod->triggerEvent("playerDeath", array($this->guid, $killer, $weapon, $damage, $bodypart));
        }
    }

    /**
     * notice that the player killed someone
     *
     * @param string $type killtype like teamkill or kill
     * @param string $victim who get killed by the player
     * @param string $weapon the weapon name which the player killed
     * @param int $damage the damage what the player dealed
     * @param string $bodypart the body part
     */
    public function kill($type, $victim, $weapon, $damage, $bodypart) {
        $this->lastvictim = $victim;
        $this->lastweaponkill = $weapon;
        $this->lastkillwastk = false;

        if ($type == "kill") {
            $this->kills ++;
            $this->damagegiven += $damage;
            if ($weapon[1] == "MOD_HEAD_SHOT") {
                $this->headshots ++;
            }
            $this->mod->triggerEvent("playerKill", array($this->guid, $victim, $weapon, $damage, $bodypart));
        } elseif ($type == "teamkill") {
            $this->teamdamagegiven += $damage;
            $this->teamkills ++;
            $this->lastkillwastk = true;
            $this->mod->triggerEvent("playerTeamKill", array($this->guid, $victim, $weapon, $damage, $bodypart));
        }
    }

    /**
     * notice that the player get damage
     *
     * @param string $type like teamdamage,damage or selfdamage
     * @param string $attacker the attaker who dealed the damage
     * @param string $weapon the weapon which the damage was dealed
     * @param int $damage the damage that was dealed
     * @param string $bodypart the part
     */
    public function damageTaken($type, $attacker, $weapon, $damage, $bodypart) {
        $this->damagetaken += $damage;
        if ($type == "teamdamage") {
            $this->mod->triggerEvent("playerTeamDamageTaken", array($this->guid, $attacker, $weapon, $damage, $bodypart));
        } elseif ($type == "damage") {
            $this->mod->triggerEvent("playerDamageTaken", array($this->guid, $attacker, $weapon, $damage, $bodypart));
        } elseif ($type == "selfdamage") {
            $this->mod->triggerEvent("playerSelfDamage", array($this->guid, $weapon, $damage, $bodypart));
        }
    }

    /**
     * notice that the player give damage
     *
     * @param string $type like teamdamage or damage
     * @param string $victim the player who get the damage
     * @param string $weapon the weapon which the damage was dealed
     * @param int $damage the damage that was dealed
     * @param string $bodypart the part
     */
    public function damageGiven($type, $victim, $weapon, $damage, $bodypart) {
        if ($type == "teamdamage") {
            $this->teamdamagegiven += $damage;
            $this->mod->triggerEvent("playerTeamDamageGiven", array($this->guid, $victim, $weapon, $damage, $bodypart));
        } elseif ($type == "damage") {
            $this->damagegiven += $damage;
            $this->mod->triggerEvent("playerDamageGiven", array($this->guid, $victim, $weapon, $damage, $bodypart));
        }
    }

    /**
     * notice how much damage the vehicle has given
     *
     * @param string $type like teamdamage or damage
     * @param string $entityid the entityid
     * @param string $weapon the weapon which the damage was dealed
     * @param int $damage the damage that was dealed
     * @param string $bodypart the body part
     */
    public function vehicleDamageGiven($type, $entityid, $weapon, $damage, $bodypart) {
        if ($type == "teamdamage") {
            $this->teamvehicledamagegiven += $damage;
            $this->mod->triggerEvent("playerTeamVehicleDamageGiven", array($this->guid, $entityid, $weapon, $damage, $bodypart));
        } elseif ($type == "damage") {
            $this->vehicledamagegiven += $damage;
            $this->mod->triggerEvent("playerVehicleDamageGiven", array($this->guid, $entityid, $weapon, $damage, $bodypart));
        }
    }

    /**
     * notice how mauch damage the actor has given
     *
     * @param string $type like teamdamage or damage
     * @param string $entityid the entityid
     * @param string $weapon the weapon which the damage was dealed
     * @param int $damage the damage that was dealed
     * @param string $bodypart the body part
     */
    public function actorsDamageGiven($type, $entityid, $weapon, $damage, $bodypart) {
        if ($type == "teamdamage") {
            $this->teamactorsdamagegiven += $damage;
            $this->mod->triggerEvent("playerTeamActorsDamageGiven", array($this->guid, $entityid, $weapon, $damage, $bodypart));
        } elseif ($type == "damage") {
            $this->actorsdamagegiven += $damage;
            $this->mod->triggerEvent("playerActorsDamageGiven", array($this->guid, $entityid, $weapon, $damage, $bodypart));
        }
    }
}
