<?php
/**
 * ManuAdminMod
 *
 * This is a powerful and platform independent software for administrating game servers of various kinds.
 * If you need help with installing or using this software, please visit our website at: www.ManuAdminMod.de
 * If you want to obtain additional permissions extending the license, contact us at: Webmaster@ManuAdminMod.de
 *
 * @copyright www.ManuAdminMod.de
 * @license http://www.creativecommons.org/licenses/by-nc-nd/4.0/ Creative Commons BY-NC-ND 4.0
 * @version 1.0.0-Beta+7
 * */

namespace MAM\Daemon\Engine\Quake3\Rcon;

use MAM\Daemon;

/**
 * Basic Rcon Commands
 *
 * @author Mirko911 <mirko911@manuadminmod.de>
 */
class RconCommands {
    /**
     * Class for mam related stuff
     *
     * @var \MAM\Daemon\Core\Mod;
     */
    protected $mod;
    /**
     * Class for Remote Commands
     *
     * @var \MAM\Daemon\Engine\Quake3\Rcon\Rcon
     */
    protected $rcon;
    /**
     * Class for Logging
     *
     * @var \MAM\Daemon\Libraries\Logging\LogHandler
     */
    protected $logging;

    /**
     * Constructor
     */
    public function __construct() {
        $registry = \MAM\Daemon\Core\Registry::getInstance();
        $this->mod = $registry->mod;
        $this->logging = $registry->logging;

        if (DEBUG) {
            $this->rcon = new \MAM\Daemon\Engine\Quake3\Rcon\RconLog($this->mod->getCV("connection", "ip"), $this->mod->getCV("connection", "port"));
        } else {
            $this->rcon = new \MAM\Daemon\Engine\Quake3\Rcon\Rcon($this->mod->getCV("connection", "ip"), $this->mod->getCV("connection", "port"));
        }

        $this->rcon->setRconpassword($this->mod->getCV("connection", "password"));

        $cod4x_test = trim($this->rcon('"_CoD4 X Creator"', true));
        $registry->iceops = $cod4x_test === '' || strpos($cod4x_test, 'Unknown command "_CoD4 X Creator"') === 0 ? false : true;
        $fastrcon = ($this->mod->getCV("quake3", "fastrcon") && $this->mod->getGame() == 'cod4') ? true : false;
        $this->rcon->changeRconSpeed($fastrcon);
    }

    /**
     * Sets (set) a Dvar via rcon
     *
     * This function doesn't return a message
     *
     * @param string $dvar
     * @param string $value
     */
    public function rconHide($dvar, $value) {
        $this->rcon->rcon("set $dvar \"$value\"");
    }

    /**
     * Gets a server's Dvar via rcon.
     * @param string $dvar
     * @return boolean|string false if an error occurs, otherwise the dvar's value
     */
    public function rconGetDvar($dvar) {
        //"g_GameType" is: "war^7" default: "war^7"
        //Domain is any text


        $response = explode("\n", $this->rcon->rcon($dvar));
        $pattern = '|^"' . $dvar . '" is: "(.*)\^7" default: "(.*)\^7"$|i';
        if (preg_match($pattern, trim($response[0]), $subpatterns)) {
            return $subpatterns[1];
        }
        return false;
    }

    /**
     * Sets (set) a Dvar via rcon.
     * @param string $dvar
     * @param string $value
     */
    public function rconSetDvar($dvar, $value) {
        $this->rcon->rcon("set $dvar \"$value\"");
        $this->logging->info("Set Dvar $dvar to '$value'");
    }

    /**
     * Sets (sets) a Dvar via rcon.
     *
     * @param string $dvar
     * @param string $value
     */
    public function rconSetsDvar($dvar, $value) {
        $this->rcon->rcon("sets $dvar $value");
        $this->logging->info("Sets Dvar $dvar to '$value'");
    }

    /**
     * Says something via rcon. Use {{br}} or \n for new line.
     *
     * @param string $msg
     */
    public function rconSay($msg) {
        $msg = utf8_decode($msg);
        $msg = str_replace("{{br}}", "\n", $msg);
        $msg = wordwrap($msg, 140, "\n", true);
        $this->mod->triggerEvent("rconSay", $msg);
        $split = explode("\n", $msg);
        foreach ($split as $value) {
            $this->rcon->rcon("say $value");
        }
    }

    /**
     * Change map (and gametype) via rcon.
     *
     * @param string $map
     * @param string $gametype
     * @return boolean true if map- and gametypechange is successful
     */
    public function rconChangeMap($map, $gametype = false) {
        if ($gametype != false && $gametype != $this->mod->getGametype()) {
            $this->rconSetGametype($gametype);
        }
        if (!$this->rconMap($map)) {
            //If mapchange failed set back gametype
            if ($gametype != false && $gametype != $this->mod->getGametype()) {
                $this->rconSetGametype($this->mod->getGametype());
            }
            return false;
        }
        return true;
    }

    /**
     * Sets gametype via rcon.
     *
     * @param string $gametype
     */
    public function rconSetGametype($gametype) {
        $this->rconSetDvar("g_gametype", $gametype);
    }

    /**
     * Sets map via rcon.
     *
     * @param string $map
     * @return boolean true if mapchange is successful
     */
    public function rconMap($map) {
        $return = $this->rcon->rcon("map $map");
        if (strpos($return, "Error: Can't find map \"$map\".") === false) {
            $this->logging->info("Map was changed to: $map");
            return true;
        }
        return false;
    }

    /**
     * Builds and returns a player array with rcon command "status".
     * @param string $map (call-by-reference) contains the mapname
     * @return array
     */
    public function rconPlayerList(&$map = NULL) {
        $return = array();

        //get playerlist by rcon
        $list = $this->rcon->rcon("status");

        $list = explode("\n", $list);

        foreach ($list as $value) {
            $pattern = '#^\s*(\d+)\s+(-?\d+)\s+(\d+)\s+([a-fA-F0-9]{16,32}|\d+) (.+?)\s+(\d+) (\d+\.\d+\.\d+\.\d+):(\-?\d+)\s+(\-?\d+)\s+(\d+)$#';
            // 1:PID 2:score 3:ping 4:guid 5:name 6:lastmsg 7:IP 8:port 9:qport 10:rate
            if (preg_match($pattern, trim($value), $subpatterns)) {
                $return[] = array(
                    "pid" => $subpatterns[1],
                    "score" => $subpatterns[2],
                    "ping" => $subpatterns[3],
                    "guid" => $subpatterns[4],
                    "name" => $subpatterns[5],
                    "lastmsg" => $subpatterns[6],
                    "ip" => $subpatterns[7],
                    "port" => $subpatterns[8],
                    "qport" => $subpatterns[9],
                    "rate" => $subpatterns[10],
                );
            }
            $pattern = '|^map: ([a-z0-9._]+)\s*$|i';
            if (preg_match($pattern, trim($value), $subpatterns)) {
                $map = $subpatterns[1];
            }
        }

        return $return;
    }

    /**
     * Sets nextmap via rcon.
     *
     * @param string $map
     * @param string $gametype
     * @param boolean $continueMapCylce Set to false if current nextmap shall be overwritten.
     * @return boolean
     */
    public function rconSetNextMap($map, $gametype, $continueMapCylce = true) {
        $nextmap = "gametype " . $gametype . " map " . $map;
        if ($continueMapCylce) {
            $rotation = $this->rconGetNextMap();
            $nextmap .= " gametype " . $rotation[1] . " map " . $rotation[0];
        }
        $this->rconSetDvar("sv_maprotationcurrent", $nextmap);
        return true;
    }

    /**
     * Runs a map restart
     * @param boolean $complete If true, a complete map restart is performed.
     */
    public function rconMapRestart($complete = false) {
        if ($complete) {
            $this->logging->info("Performing complete map restart");
            $this->rcon->rcon("map_restart");
        } else {
            $this->logging->info("Performing fast map restart");
            $this->rcon->rcon("fast_restart");
        }
    }

    /**
     * Shutdown the gameserver.
     * @param int $delay
     */
    public function rconKillServer($delay = 3) {
        $this->rconSay($this->getLngString("killserver"));
        for ($i = $delay; $i >= 1; $i --) {
            $this->rconSay($i);
        }
        $this->rcon->rcon("quit");
        $this->logging->info("Gameserver was shut down");
    }

    /**
     * Load nextmap via rcon map_rotate.
     */
    public function rconMapRotate() {
        $this->rcon->rcon("map_rotate");
        $this->logging->info("Map was rotated");
    }

    /**
     * Returns the nextmap.
     * @return array 0=>map, 1=>gametype and "map"=>map, "gametype"=>gametype
     */
    public function rconGetNextMap() {
        $return = $this->rconDvarList("sv_maprotation*");
        $current = $return["sv_maprotationcurrent"];
        $mapcycle = $return["sv_maprotation"];
        // > gametype sd map mp_bloc [...](^7)?<
        $pattern = '|^\s*(gametype\s+([a-z]+)\s+)?map\s+([a-z0-9_.-]+)\s*.*|i';
        if (preg_match($pattern, $current, $subpatterns)) {
            if (empty($subpatterns[2])) {
                $gametype = $this->mod->getGametype();
            } else {
                $gametype = $subpatterns[2];
            }
            $next = array("map" => $subpatterns[3], "gametype" => $gametype, 0 => $subpatterns[3], 1 => $gametype);
        } elseif (preg_match($pattern, $mapcycle, $subpatterns)) {
            if (empty($subpatterns[2])) {
                $gametype = $this->mod->getGametype();
            } else {
                $gametype = $subpatterns[2];
            }
            $next = array("map" => $subpatterns[3], "gametype" => $gametype, 0 => $subpatterns[3], 1 => $gametype);
        } else {
            $next = array("map" => "Unknown", "gametype" => "Unknown", 0 => "Unknown", 1 => "Unknown");
        }
        return $next;
    }

    /**
     * Kicks all player from server.
     */
    public function rconKickAll() {
        $this->rcon->rcon("kick all");
        $this->logging->info("Everyone got kicked from the server");
    }

    /**
     * Returns an array with the dvarlist from the server
     * @param string $str
     * @return array
     */
    public function rconDvarList($str = "") {
        $result = $this->rcon->rcon("dvarlist $str");
        $lines = explode("\n", $result);
        $lines = array_map("rtrim", $lines);

        $dvars = array();
        $pattern = '|^[A-Z ]{7} (\w+) "(.*)"$|i';

        foreach ($lines as $value) {
            if (preg_match($pattern, $value, $subpatterns)) {
                $dvars[strtolower($subpatterns[1])] = $subpatterns[2];
            }
        }

        return $dvars;
    }

    /**
     * Changes the map to $map with "devmap"!
     * (The devmap command allows you to perform cheats on your server, so you can get things such as god mode, health, ammo, noclip etc.)
     * @param string $map
     * @return boolean true if successful, otherwise false
     */
    public function rconDevMap($map) {
        $return = $this->rcon->rcon("devmap $map");
        if (strpos($return, "Error: Can't find map \"$map\".") === false) {
            $this->logging->info("Map was changed to: $map with enabled cheats");
            return true;
        }
        return false;
    }

    /**
     * Execute file on server.
     * @param string $file
     * @return boolean true if successfull, otherwise false
     */
    public function rconExec($file) {
        $result = $this->rcon->rcon("exec $file");
        if (stripos($result, "Error: couldn't exec $file") !== false) {
            return false;
        }
        return true;
    }

    /**
     * Reconnect to server
     *
     * @return mixed
     */
    public function rconConnect() {
        return $this->rcon->connect();
    }

    /**
     * Executes any rcon command.
     * @param string $command
     * @param boolean $hidden Hide the logprint
     * @return string response
     */
    public function rcon($command, $hidden = false) {
        if (!$hidden) {
            $this->logging->info("RCON command executed: '$command'");
        }
        return $this->rcon->rcon($command);
    }

    /**
     * Says private message to a player
     *
     * @param int $pid PID of the player you want to whisper
     * @param string $msg Message
     */
    public function rconTell($pid, $msg) {
        $msg = utf8_decode($msg);
        $this->rcon->rcon("tell " . $pid . " " . $msg);
    }
}
