<?php
/**
 * ManuAdminMod
 *
 * This is a powerful and platform independent software for administrating game servers of various kinds.
 * If you need help with installing or using this software, please visit our website at: www.ManuAdminMod.de
 * If you want to obtain additional permissions extending the license, contact us at: Webmaster@ManuAdminMod.de
 *
 * @copyright www.ManuAdminMod.de
 * @license http://www.creativecommons.org/licenses/by-nc-nd/4.0/ Creative Commons BY-NC-ND 4.0
 * @version 1.0.0-Beta+7
 * */

namespace MAM\Daemon\Libraries\Punishment;

use MAM\Daemon\Core\Registry;

/**
 * Class BanHelper stores all bans in a DB to unban a player
 *
 * @author Master of Little <masteroflittle@manuadminmod.de>
 */
class BanHelper {
    /**
     * Database object
     *
     * @var \MAM\Daemon\Libraries\Database\Database
     */
    private $database;
    /**
     * Mod
     *
     * @var \MAM\Daemon\Core\Mod
     */
    private $mod;
    /**
     * Players
     *
     * @var \MAM\Daemon\Core\Player
     */
    private $players;
    /**
     * Logging
     *
     * @var \MAM\Daemon\Libraries\Logging\LogHandler
     */
    private $logging;

    /**
     * Constructor for the BanHelper class
     */
    public function __construct() {
        $registry = Registry::getInstance();
        $this->mod = $registry->mod;
        $this->players = & $registry->players;
        $this->logging = $registry->logging;
        $this->database = $registry->database;

        $this->clearExpired();

        $registry->banhelper = $this;
    }

    /**
     * Returns a string if object is printed
     *
     * Notice: all plugins which where moved to core needs a __tostring function
     * to get it working with the permissions system. Therefore the return of __tostring must me the
     * namespace like deamon, punishment, permissions ...
     *
     * @see http://php.net/manual/en/language.oop5.magic.php#object.tostring
     * @return string
     */
    public function __toString() {
        return "BanHelper";
    }

    /**
     * Writes ban to DB
     * @param string $guid
     * @param string $pbguid
     * @param string $name
     * @param string $reason
     * @param int $expire 0 = ban does not expire
     * @param string $type
     */
    public function writeBan($guid, $pbguid, $name, $reason, $expire = 0, $type = 'guid') {
        $check = (array) $this->getBan('guid', $guid);
        if (!empty($check)) {
            $this->database->delete('punishmentBanHelper', 'user', $check[0]->user);
        }
        if (!$this->database->insert('punishmentBanHelper', array('user' => $this->players[$guid]->getUserId(), 'guid' => $guid, 'pbguid' => $pbguid, 'name' => $name, 'ip' => $this->players[$guid]->getIP(), 'reason' => $reason, 'expire' => $expire, 'type' => $type))) {
            $this->logging->warning($this . ': Can \'t create ban for user ' . $this->players[$guid]->getUserId());
        }
    }

    /**
     * Delets a ban from banHelper Database
     *
     * @param int $id ID of the ban
     */
    function deleteBan($id) {
        $this->database->delete('punishmentBanHelper', 'user', $id);
    }

    /**
     * Get ban from DB
     * @param string $search
     * @param $value
     * @return array of db-Objects
     */
    public function getBan($search, $value) {
        $this->clearExpired();
        $result = $this->database->get('punishmentBanHelper', "SELECT * FROM <TABLE> WHERE $search LIKE :val", array(':val' => "%" . $value . "%"));
        return $result;
    }

    /**
     * Get all bans from DB
     * @return Object
     */
    public function getAllBans() {
        $this->clearExpired();
        $result = $this->database->get('punishmentBanHelper');
        return $result;
    }

    /**
     * Clear the expired bans
     */
    private function clearExpired() {
        $this->database->exec('DELETE FROM <PFX>punishmentBanHelper WHERE expire != 0 AND expire < ' . time());
    }
}
