<?php
/**
 * ManuAdminMod
 *
 * This is a powerful and platform independent software for administrating game servers of various kinds.
 * If you need help with installing or using this software, please visit our website at: www.ManuAdminMod.de
 * If you want to obtain additional permissions extending the license, contact us at: Webmaster@ManuAdminMod.de
 *
 * @copyright www.ManuAdminMod.de
 * @license http://www.creativecommons.org/licenses/by-nc-nd/4.0/ Creative Commons BY-NC-ND 4.0
 * @version 1.0.0-Beta+7
 **/

namespace MAM\Daemon\Libraries\Punishment;

use MAM\Daemon\Core\Registry;

/**
 * Class for punishment commands
 *
 * @author Master of Little <masteroflittle@manuadminmod.de>
 */
class Commands {
    /**
     * Class for mam related stuff
     *
     * @var \MAM\Daemon\Engine\Quake3\Mod
     * @var \MAM\Daemon\Engine\Frostbite3\Mod
     * @var \MAM\Daemon\Core\Mod
     */
    private $mod;
    /**
     * Class for Remote Commands
     *
     * @var \MAM\Daemon\Engine\Quake3\Rcon\RconCommands
     * @var \MAM\Daemon\Engine\Frostbite3\Rcon\RconCommands
     */
    private $rcon;
    /**
     * Class for player related stuff
     *
     * @var \MAM\Daemon\Core\Player
     * @var \MAM\Daemon\Engine\Frostbite3\Player
     * @var \MAM\Daemon\Engine\Quake3\Player
     *
     */
    private $players;
    /**
     * Class for Logging
     *
     * @var \MAM\Daemon\Libraries\Logging\LogHandler
     */
    private $logging;
    /**
     * Permissions Class
     *
     * @var \MAM\Daemon\Libraries\Permissions\Permissions
     */
    private $permissions;
    /**
     * Punishment Class
     *
     * @var \MAM\Daemon\Libraries\Punishment\Punishment
     */
    private $punishment;
    /**
     * array of punishment commands
     * @var array
     */
    private $commands = array();
    /**
     * Ban helper
     * @var \MAM\Daemon\Libraries\Punishment\BanHelper
     */
    private $banhelper;

    /**
     * Constructor for punishment commands class
     */
    public function __construct() {
        $registry = Registry::getInstance();
        $this->mod = $registry->mod;
        $this->players = & $registry->players;
        $this->rcon = $registry->rcon;
        $this->logging = $registry->logging;
        $this->permissions = $registry->permissions;
        $this->punishment = $registry->punishment;
        $this->banhelper = $registry->banhelper;

        $this->registerCommand('warn');
        $this->registerCommand('kick');
        $this->registerCommand('ban');
        $this->registerCommand('tempBan');
        $this->registerCommand('getScore');
        $this->registerCommand('getLastPunishment');
        $this->registerCommand('unBan');

        $this->mod->registerCommand('punishment', false, 'punishmentCommand', $this);
    }

    /**
     * Returns a string if object is printed
     *
     * Notice: all plugins which where moved to core needs a __tostring function
     * to get it working with the permissions system. Therefore the return of __tostring must me the
     * namespace like deamon, punishment, permissions ...
     *
     * @see http://php.net/manual/en/language.oop5.magic.php#object.tostring
     * @return string
     */
    public function __toString() {
        return "punishment";
    }

    /**
     * Register a punishment command
     * @param string $command
     */
    private function registerCommand($command) {
        $this->commands[strtolower($command)] = $command;
    }

    /**
     * Main function for punishment commands execution
     * @param   string      $guid       Guid of executing player
     * @param   string[]    $parameters The chatline splitted by " " without !command
     */
    public function punishmentCommand($guid, $parameters) {

        $command = strtolower(array_shift($parameters));
        if (!array_key_exists($command, $this->commands)) {
            $this->players[$guid]->say($this->mod->getLngString('punishmentNotFound', '<COMMAND>', $command));
            return;
        }

        if (!$this->permissions->hasPermissions($guid, 'punishment.' . $this->commands[$command] . '.canexecute')) {
            $this->players[$guid]->say($this->mod->getLngString("morePowerNeeded"));
            return;
        }
        $command = $this->commands[$command];
        $this->$command($guid, $parameters);
    }

    /**
     * Returns the Score of a Player to the executing player
     * @param   string      $guid       Guid of executing player
     * @param   string[]    $parameters The chatline splitted by " " without !command
     */
    public function getScore($guid, $parameters) {
        if (empty($parameters)) {
            $this->mod->printHelp('getScore', $guid);
            return;
        }
        $find = $this->mod->findPlayerGuid($parameters[0]);
        if (!$find) {
            $this->players[$guid]->say($this->mod->getLngString('playerNotFound', array("<SEARCH>"), array($parameters[0])));
            return;
        }
        $group = $this->players[$find]->getGroup();
        if (!$this->permissions->hasPermissions($guid, 'punishment.getscore.' . $group)) {
            $this->players[$guid]->say($this->mod->getLngString("morePowerNeeded"));
            return;
        }
        $this->players[$guid]->say($this->mod->getLngString("getScore", array('<SCORE>'), array($this->players[$find]->getPunishmentScore())));
    }

    /**
     * Returns the Score of a Player to the executing player
     * @param   string      $guid       Guid of executing player
     * @param   string[]    $parameters The chatline splitted by " " without !command
     */
    public function getLastPunishment($guid, $parameters) {
        if (empty($parameters)) {
            $this->mod->printHelp('getLastPunishment', $guid);
            return;
        }
        $find = $this->mod->findPlayerGuid($parameters[0]);
        if (!$find) {
            $this->players[$guid]->say($this->mod->getLngString('playerNotFound', array("<SEARCH>"), array($parameters[0])));
            return;
        }
        $group = $this->players[$find]->getGroup();
        if (!$this->permissions->hasPermissions($guid, 'punishment.getlastpunishment.' . $group)) {
            $this->players[$guid]->say($this->mod->getLngString("morePowerNeeded"));
            return;
        }
        $this->players[$guid]->say($this->mod->getLngString("getLastPunishment", array('<SCORE>'), array($this->players[$find]->getLastPunishment())));
    }

    /**
     * Kicks a player
     *
     * @param   string      $guid       Guid of executing player
     * @param   string[]    $parameters The chatline splitted by " " without !command
     */
    public function kick($guid, $parameters) {
        if (empty($parameters)) {
            $this->mod->printHelp('kick', $guid);
            return;
        }
        $parastring = implode(" ", $parameters);
        if (strpos($parastring, "\\") !== false) {
            list($player, $parameters) = explode("\\", $parastring, 2);
            $player = trim($player);
            $parameters = array(trim($parameters));
        } else {
            $player = array_shift($parameters);
        }
        $tokick = $this->mod->findPlayerGuid($player);

        if (!$tokick) {
            $this->players[$guid]->say($this->mod->getLngString("playerNotFound", array("<SEARCH>"), array($player)));
        } else {
            $group = $this->players[$tokick]->getGroup();
            if (!$this->permissions->hasPermissions($guid, 'punishment.kick.' . $group)) {
                $this->players[$guid]->say($this->mod->getLngString("morePowerNeeded"));
                return;
            }

            if (!empty($parameters)) {
                $reason = implode(" ", $parameters);
                $this->mod->findReason($reason);
            } else {
                //Check if you force your admins to enter a reason
                if ($this->mod->getCV("punishment", "forcereason")) {
                    $this->players[$guid]->say($this->mod->getLngString("forceReason"));
                    return;
                }
                $reason = false;
            }
            if ($this->players[$tokick]->kick($reason, $guid)) {
                if (!$this->mod->getCV("punishment", "announce")) {
                    $this->players[$guid]->say($this->mod->getLngString("playerKicked", array("<PLAYER>"), array($this->players[$tokick]->getName())));
                }
            } else {
                $this->players[$guid]->say($this->mod->getLngString("anyError"));
            }
        }
    }

    /**
     * Bans a player
     *
     * @param   string      $guid       Guid of executing player
     * @param   string[]    $parameters The chatline splitted by " " without !command
     */
    public function ban($guid, $parameters) {
        if (empty($parameters)) {
            $this->mod->printHelp('ban', $guid);
            return;
        }
        $parastring = implode(" ", $parameters);
        if (strpos($parastring, "\\") !== false) {
            list($player, $parameters) = explode("\\", $parastring);
            $player = trim($player);
            $parameters = array(trim($parameters));
        } else {
            $player = array_shift($parameters);
        }

        $tokick = $this->mod->findPlayerGuid($player);

        if (!$tokick) {
            $this->players[$guid]->say($this->mod->getLngString("playerNotFound", array("<SEARCH>"), array($player)));
        } else {
            $group = $this->players[$tokick]->getGroup();
            if (!$this->permissions->hasPermissions($guid, 'punishment.ban.' . $group)) {
                $this->players[$guid]->say($this->mod->getLngString("morePowerNeeded"));
                return;
            }

            if (!empty($parameters)) {
                $reason = implode(" ", $parameters);
                $this->mod->findReason($reason);
            } else {
                if ($this->mod->getCV("punishment", "forcereason")) {
                    $this->players[$guid]->say($this->mod->getLngString("forceReason"));
                    return;
                }
                $reason = false;
            }
            if ($this->players[$tokick]->ban($reason, $guid)) {
                if (!$this->mod->getCV("punishment", "announce")) {
                    $this->players[$guid]->say($this->mod->getLngString("playerBanned", array("<PLAYER>"), array($this->players[$tokick]->getName())));
                }
            } else {
                $this->players[$guid]->say($this->mod->getLngString("anyError"));
            }
        }
    }

    /**
     * TempBans a player
     *
     * @param   string      $guid       Guid of executing player
     * @param   string[]    $parameters The chatline splitted by " " without !command
     */
    public function tempBan($guid, $parameters) {
        if (empty($parameters)) {
            $this->mod->printHelp('tempban', $guid);
            return;
        }
        $parastring = implode(" ", $parameters);
        if (strpos($parastring, "\\") !== false) {
            list($player, $parameters) = explode("\\", $parastring);
            $player = trim($player);
            $parameters = array(trim($parameters));
        } else {
            $player = array_shift($parameters);
        }

        $tokick = $this->mod->findPlayerGuid($player);

        if (!$tokick) {
            $this->players[$guid]->say($this->mod->getLngString("playerNotFound", array("<SEARCH>"), array($player)));
        } else {

            $group = $this->players[$tokick]->getGroup();
            if (!$this->permissions->hasPermissions($guid, 'punishment.tempban.' . $group)) {
                $this->players[$guid]->say($this->mod->getLngString("morePowerNeeded"));
                return;
            }

            if (!empty($parameters)) {
                if (is_numeric($parameters[0]) && $parameters[0] > 0) {
                    $time = array_shift($parameters);
                    if ($time > $this->mod->getCV("punishment", "maxtempbandur")) {
                        $time = $this->mod->getCV("punishment", "maxtempbandur");
                    }
                } else {
                    $time = false;
                }
                if (!empty($parameters)) {
                    $reason = implode(" ", $parameters);
                    $this->mod->findReason($reason);
                } else {
                    if ($this->mod->getCV("punishment", "forcereason")) {
                        $this->players[$guid]->say($this->mod->getLngString("forceReason"));
                        return;
                    }
                    $reason = false;
                }
            } else {
                $reason = false;
                $time = false;
            }
            if ($this->players[$tokick]->tempBan($reason, $guid, $time)) {
                if (!$this->mod->getCV("punishment", "announce")) {
                    $this->players[$guid]->say($this->mod->getLngString("playerTempBanned", array("<PLAYER>"), array($this->players[$tokick]->getName())));
                }
            } else {
                $this->players[$guid]->say($this->mod->getLngString("anyError"));
            }
        }
    }

    /**
     * warns a player and add 1 to his punishment score
     * @param   string      $guid       Guid of executing player
     * @param   string[]    $parameters The chatline splitted by " " without !command
     */
    public function warn($guid, $parameters) {
        if (empty($parameters)) {
            $this->mod->printHelp('warn', $guid);
            return;
        }
        $parastring = implode(" ", $parameters);
        if (strpos($parastring, "\\") !== false) {
            list($player, $parameters) = explode("\\", $parastring, 2);
            $player = trim($player);
            $parameters = array(trim($parameters));
        } else {
            $player = array_shift($parameters);
        }
        $towarn = $this->mod->findPlayerGuid($player);

        if (!$towarn) {
            $this->players[$guid]->say($this->mod->getLngString("playerNotFound", array("<SEARCH>"), array($player)));
            return;
        }

        $reason = implode(" ", $parameters);
        if (!$reason && $this->mod->getCV("punishment", "forcereason") && $guid != false) {
            $this->players[$guid]->say($this->mod->getLngString("forceReason"));
            return;
        } elseif (!$reason) {
            $reason = $this->mod->getCV("daemon", "defaultkickreason");
        }
        $this->mod->findReason($reason);

        $this->players[$towarn]->punishmentScoreAdd($this->mod->getCV("punishment", "pointsforwarn"));

        $action = $this->punishment->checkScore($this->players[$towarn]->getPunishmentScore(), 1);
        if ($action !== false) {
            $this->punishment->punish($towarn, $action, 1, array("reason" => $reason, "kicker" => $guid));
        }
    }

    /**
     * Unban a player
     *
     * @param   string      $guid       Guid of executing player
     * @param   string[]    $parameters The chatline splitted by " " without !command
     */
    public function unBan($guid, $parameters) {
        if (empty($parameters)) {
            $this->mod->printHelp('unBan', $guid);
            return;
        }
        $what = $parameters[0];
        unset($parameters[0]);
        $search = implode(" ", $parameters);
        switch ($what) {
            case 'guid':
                $array = $this->banhelper->getBan('guid', $search);
                if (empty($array)) {
                    $this->players[$guid]->say($this->mod->getLngString("unBanGuidNotFound", array("<SEARCH>"), array($search)));
                } else {
                    if (count($array) > 1) {
                        $this->players[$guid]->say($this->mod->getLngString("unBanToMuch", array("<AMOUNT>"), array($search)));
                    } else {
                        $object = $array[0];
                        if ($object->type != 'pb') {
                            $this->unBanCmd($object, false);
                        } else {
                            $this->unBanCmd($object, true);
                        }
                        $this->banhelper->deleteBan($object->user);
                        $this->logging->notice("Removed ban for $object->name($object->guid)");
                        $this->players[$guid]->say($this->mod->getLngString("banRemoved", array("<AMOUNT>"), array($search)));
                    }
                }
                break;
            case 'ip':
                $array = $this->banhelper->getBan('name', $search);
                if (empty($array)) {
                    $this->players[$guid]->say($this->mod->getLngString("unBanIpNotFound", array("<SEARCH>"), array($search)));
                    if (count($array) > 1) {
                        $this->players[$guid]->say($this->mod->getLngString("unBanToMuch", array("<AMOUNT>"), array($search)));
                    } else {
                        $object = $array[0];
                        if ($object->type != 'pb') {
                            $this->unBanCmd($object, false);
                        } else {
                            $this->unBanCmd($object, true);
                        }
                        $this->banhelper->deleteBan($object->user);
                        $this->logging->notice("Removed ban for $object->ip($object->guid)");
                        $this->players[$guid]->say($this->mod->getLngString("banRemoved", array("<AMOUNT>"), array($search)));
                    }
                }
                break;
            default:
                $array = $this->banhelper->getBan('name', $search);
                if (empty($array)) {
                    $this->players[$guid]->say($this->mod->getLngString("unBanNameNotFound", array("<SEARCH>"), array($search)));
                } else {
                    if (count($array) > 1) {
                        $this->players[$guid]->say($this->mod->getLngString("unBanToMuch", array("<AMOUNT>"), array($search)));
                    } else {
                        $object = $array[0];
                        if ($object->type != 'pb') {
                            $this->unBanCmd($object, false);
                        } else {
                            $this->unBanCmd($object, true);
                        }
                        $this->banhelper->deleteBan($object->user);
                        $this->logging->notice("Removed ban for $object->name($object->guid)");
                        $this->players[$guid]->say($this->mod->getLngString("banRemoved", array("<AMOUNT>"), array($search)));
                    }
                }
                break;
        }
    }

    /**
     * Sends the rcon command to unban a player
     *
     * @param ODO-Object $object
     * @param bool $pb If ban was with pb unban him with it
     */
    private function unBanCmd($object, $pb = false) {
        if ($pb === false) {
            if ($this->mod->getEngine() == "frostbite3") {
                $this->rcon->rcon(array('banList.remove', $object->type, $object->{$object->type}));
                $this->rcon->rcon('banList.save');
            } else {
                if (Registry::getInstance()->iceops) {
                    $this->rcon->rcon("unban $object->guid", true);
                } else {
                    $this->rcon->rcon("unbanUser $object->name", true);
                }
            }
        } else {
            if ($this->mod->getEngine() == "frostbite3") {
                $this->rcon->rcon(array('punkbuster.pb_sv_command', 'PB_SV_UnBanGuid ' . $object->pbguid));
            } else {
                $this->rcon->rcon('PB_SV_UnBanGuid ' . $object->pbguid, true);
            }
        }
    }
}
