<?php
/**
 * ManuAdminMod
 *
 * This is a powerful and platform independent software for administrating game servers of various kinds.
 * If you need help with installing or using this software, please visit our website at: www.ManuAdminMod.de
 * If you want to obtain additional permissions extending the license, contact us at: Webmaster@ManuAdminMod.de
 *
 * @copyright www.ManuAdminMod.de
 * @license http://www.creativecommons.org/licenses/by-nc-nd/4.0/ Creative Commons BY-NC-ND 4.0
 * @version 1.0.0-Beta+7
 * */
/*

  enabled = 1
  type = 1 // 1=Maps werden vorgeschlagen; // 2=MW2-Modus, nur !skip is m�glich
  maps = ''
  mapcount = 4
  duration = 300
  announce = 45
  notlastmaps = 3
  votestoskip = 5% //Absolute Anzahl der Spieler oder prozentualer Anteil (wenn % verwendet wird)
  gametypes
  notlastgt = 1

 */

namespace MAM\Plugins\Mapvote;

use MAM\Daemon\Core\Utils;

/**
 * Mapvote Class
 *
 * The mapvote plugin
 *
 * @author manu
 * @license http://creativecommons.org/licenses/by-nc-sa/4.0/ Creative Commons BY-NC-SA 4.0
 * @todo Move !vmap and !skip to 1 command, because Dennis likes it
 * @todo Remove underscore in function names
 * @todo rename <maps.json> to <all>
 */
class Mapvote extends Utils {
    /**
     * Maps
     *
     * @var array $maps Maps
     */
    private $maps = array();
    /**
     * lastmaps
     *
     * @var array $lastmaps the last maps
     */
    private $lastmaps = array();
    /**
     * Chosen
     *
     * @var array $chosen
     */
    private $chosen = array();
    /**
     * Start
     *
     * @var int $start
     */
    private $start = 0;
    /**
     * last announce
     *
     * @var int $lastannounce
     */
    private $lastAnnounce = 0;
    /**
     * vote in progress
     *
     * @var boolean $voteinprogress
     */
    private $voteinprogress = false;
    /**
     * Skips
     *
     * @var array $skips
     */
    private $skips = array();
    /**
     * Gametypes
     *
     * @var array $gametypes
     */
    private $gametypes = array();
    /**
     * Voters
     *
     * @var array $voters
     */
    private $voters = array();
    /**
     * Votes
     *
     * @var array $votes
     */
    private $votes = array();

    /**
     * Inits the plugin
     *
     * This function initiates the plugin. This means that it register commands
     * default values, and events. It's important that every plugin has this function
     * Otherwise the plugin exists but can't be used
     */
    public function initPlugin() {
        parent::initPlugin();

        $this->readConfig();

        list($this->lastmaps[]) = $this->mod->getLastMaps(1);

        $this->mod->setDefaultCV('mapvote', 'enabled', false);
        $this->mod->setDefaultCV('mapvote', 'type', 2);
        $this->mod->setDefaultCV('mapvote', 'mapcount', 3);
        $this->mod->setDefaultCV('mapvote', 'duration', 300);
        $this->mod->setDefaultCV('mapvote', 'announce', 90);
        $this->mod->setDefaultCV('mapvote', 'maps', '<all>');
        $this->mod->setDefaultCV('mapvote', 'gametypes', '<all>');
        $this->mod->setDefaultCV('mapvote', 'notlastmaps', 3);
        $this->mod->setDefaultCV('mapvote', 'notlastgt', true);
        $this->mod->setDefaultCV('mapvote', 'votestoskip', '50%');
    }

    /**
     * Runs the plugin
     *
     * This function is called after all plugins were loaded and dependencies were checked.
     * In this function you can define things which you want to load, when all plugins are fine.
     *
     * If you know that you don't have any dependencies, you can remove this function.
     * Otherwise you should use it.
     *
     * In this function you should call functions like registerCommand, registerEvent, triggerEvent
     */
    public function runPlugin() {
        parent::runPlugin();

        $this->mod->registerEvent('nextMap', 'eventNextMap', $this);
        $this->mod->registerEvent('everyTime', 'eventEveryTime', $this);
        $this->mod->registerEvent('playerSay', 'eventPlayerSay', $this);

        $this->mod->registerCommand('skip', false, 'commandSkip', $this);
        $this->mod->registerCommand('vmap', '|vmap \d+|i', 'commandVmap', $this);
    }

    /**
     * executes the readConfig event.
     */
    private function readConfig() {
        if ($this->mod->getCV('randommapcycle', 'enabled') && $this->mod->getCV('mapvote', 'enabled')) {
            $this->logging->warning('Mapvote: Randommapcycle is activated; you should only use either randommapcycle or mapvote but not both');
        }
        if ($this->mod->getCV('mapvote', 'maps') == '<all>') {
            $this->maps = $this->mod->getMaps();
        } else {
            $this->maps = $this->mod->getCV('mapvote', 'maps');
        }
        if ($this->mod->getCV('mapvote', 'gametypes') == '<all>') {
            $this->gametypes = $this->mod->getGametypes();
        } else {
            $this->gametypes = $this->mod->getCV('mapvote', 'gametypes');
        }
    }

    /**
     * executes the nextMap event.
     */
    public function eventNextMap() {
        if (!$this->mod->getCV('mapvote', 'enabled')) {
            return;
        }

        $this->readConfig();

        $lastmaps = $this->mod->getLastMaps(2);

        if ($lastmaps[1] == $lastmaps[0]) {
            return;
        }

        $this->lastmaps[] = $lastmaps[0];

        $this->voteinprogress = false;

        $last_map_count = count($this->lastmaps);
        while ($last_map_count > $this->mod->getCV('mapvote', 'notlastmaps')) {
            array_shift($this->lastmaps);
        }
        $this->lastmaps = array_merge($this->lastmaps);

        if ($this->mod->getCV('mapvote', 'type') == 2) {
            $this->announceSkipPossible();
        } else {
            $this->startVote();
        }
    }

    /**
     * executes the playerSay event.
     *
     * @param array $parameters the possible parameters
     */
    public function eventPlayerSay($parameters) {
        list($guid, $msg, $executed) = $parameters;

        if (!$this->mod->getCV('mapvote', 'enabled')) {
            return;
        }

        if ($executed) {
            return;
        }

        if ($this->mod->getCV('mapvote', 'type') == 2 && strtolower(trim($this->mod->removeColor($msg))) == 'skip') {
            $this->mod->executeCommand($this->mod->getCV('daemon', 'cmdprefix') . 'skip', $guid, $executed);
        }
        $msg = explode(' ', $msg);
        if ($this->mod->getCV('mapvote', 'type') == 1 && strtolower(trim($this->mod->removeColor($msg[0]))) == 'vmap') {
            if (is_numeric($msg[1])) {
                $this->mod->executeCommand($this->mod->getCV('daemon', 'cmdprefix') . 'vmap ' . $msg[1], $guid, $executed);
            }
        }
    }

    /**
     * executes the everyTime event.
     */
    public function eventEveryTime() {
        if (!$this->mod->getCV('mapvote', 'enabled')) {
            return;
        }

        if ($this->mod->getCV('mapvote', 'type') == 2) {
            if (time() - $this->lastAnnounce >= $this->mod->getCV('mapvote', 'announce')) {
                $this->announceSkipMessage();
            }
            return;
        }

        if (!$this->voteinprogress) {
            return;
        }

        if (time() - $this->start >= $this->mod->getCV('mapvote', 'duration')) {
            $this->endVote();
        } elseif (time() - $this->lastAnnounce >= $this->mod->getCV('mapvote', 'announce')) {
            $this->announceVote();
        }
    }

    /**
     * Starts a Vote
     */
    private function startVote() {
        $this->chooseMap($this->mod->getCV('mapvote', 'mapcount'));

        $this->start = time();
        $this->voteinprogress = true;
        $this->votes = array_fill(0, $this->mod->getCV('mapvote', 'mapcount'), 0);
        $this->voters = array();

        $this->lastAnnounce = time() - $this->mod->getCV('mapvote', 'announce') + 20;
    }

    /**
     * Announces a Vote
     */
    private function announceVote() {
        $this->lastAnnounce = time();

        $mapstr = array();
        foreach ($this->chosen as $id => $map) {
            $mapstr[] = $this->mod->getLngString('mapvoteList', array('<ID>', '<MAP>', '<GAMETYPE>'), array($id + 1, $this->mod->getLongMapName($map['map']), $this->mod->getLongGametype($map['gametype'])));
        }
        $mapstr = implode('\n', $mapstr);

        $this->rcon->rconSay($this->mod->getLngString('mapvotevoteformap') . '\n$mapstr');
    }

    /**
     * Ends a Vote
     */
    private function endVote() {
        $this->voteinprogress = false;

        arsort($this->votes);
        reset($this->votes);
        $won = key($this->votes);
        $this->rcon->rconSay($this->mod->getLngString('mapvotewonvote', array('<MAP>', '<GAMETYPE>'), array($this->mod->getLongMapName($this->chosen[$won]['map']), $this->mod->getLongGametype($this->chosen[$won]['gametype']))));
        $this->rcon->rconSetNextMap($this->chosen[$won]['map'], $this->chosen[$won]['gametype'], false);
    }

    /**
     * announce if the vote is possible to skip.
     */
    private function announceSkipPossible() {
        $this->readConfig();

        $this->chooseMap(1);

        $this->rcon->rconSetNextMap($this->chosen[0]['map'], $this->chosen[0]['gametype'], false);
        $this->skips = array();
        $this->lastAnnounce = time() - $this->mod->getCV('mapvote', 'announce') + 20;
    }

    /**
     * the message on the server for the skip vote.
     */
    private function announceSkipMessage() {
        $this->rcon->rconSay($this->mod->getLngString('mapvotevoteforskip', array('<MAP>', '<GAMETYPE>'), array($this->mod->getLongMapName($this->chosen[0]['map']), $this->mod->getLongGametype($this->chosen[0]['gametype']))));
        $this->lastAnnounce = time();
    }

    /**
     * execztes the command skip.
     *
     * @param   string      $guid       Guid of executing player
     * @param   string[]    $parameters The chatline splitted by " " without !command
     */
    public function commandSkip($guid, $parameters) {
        if (!$this->mod->getCV('mapvote', 'enabled')) {
            return;
        }

        if ($this->mod->getCV('mapvote', 'type') != 2) {
            return;
        }

        if (in_array($guid, $this->skips)) {
            $this->players[$guid]->say($this->mod->getLngString('mapvotealreadyvoted'));
        } else {
            $this->skips[] = $guid;

            if (substr(trim($this->mod->getCV('mapvote', 'votestoskip')), -1) == '%') {
                $needed = round(((int) $this->mod->getCV('mapvote', 'votestoskip')) / 100 * count($this->players));
            } else {
                $needed = (int) $this->mod->getCV('mapvote', 'votestoskip');
            }

            $skippers = count($this->skips);
            $say = $this->mod->getLngString('mapvotevotedskip', array('<SKIPPERS>', '<NEEDED>', '<MAP>', '<GAMETYPE>'), array($skippers, $needed, $this->mod->getLongMapName($this->chosen[0]['map']), $this->mod->getLongGametype($this->chosen[0]['gametype'])));
            $this->rcon->rconSay($say);

            if ($skippers >= $needed) {
                $this->announceSkipPossible();
                $this->announceSkipMessage();
            }
        }
    }

    /**
     * choose a map
     *
     * @param int $count
     */
    private function chooseMap($count) {
        $maps = $this->maps;
        $gametypes = $this->gametypes;

        foreach ($maps as $key => $value) {
            if (in_array($value, $this->lastmaps)) {
                unset($maps[$key]);
            }
        }


        if ($this->mod->getCV('mapvote', 'notlastgt')) {
            $current = $this->mod->getCurrentGametype();
            foreach ($gametypes as $key => $value) {
                if ($value == $current) {
                    unset($gametypes[$key]);
                }
            }
        }

        $gametypes = array_merge($gametypes);

        if (count($gametypes) == 0) {
            $gametypes[0] = $this->mod->getCurrentGametype();
        }

        if (count($maps) < $count) {
            $this->logging->warning('Mapvote: Can\'t choose a random map because count(\$maps) < $count');
            return false;
        }

        $this->chosen = array();
        for ($i = 1; $i <= $count; $i++) {
            $maps = array_merge($maps);
            $rand = rand(0, count($maps) - 1);
            $rand2 = rand(0, count($gametypes) - 1);
            $this->chosen[] = array('map' => $maps[$rand], 'gametype' => $gametypes[$rand2]);
            unset($maps[$rand]);
        }
        return true;
    }

    /**
     * executes the command vmap
     *
     * Example: !vmap 1-3
     *
     * @param   string      $guid       Guid of executing player
     * @param   string[]    $parameters The chatline splitted by " " without !command
     */
    public function commandVmap($guid, $parameters) {
        if (!$this->mod->getCV('mapvote', 'enabled')) {
            return;
        }

        if ($this->mod->getCV('mapvote', 'type') != 1) {
            return;
        }

        $map = $parameters[0] - 1;

        if (!$this->voteinprogress) {
            return;
        }
        if (!array_key_exists($map, $this->chosen)) {
            return;
        }
        if (in_array($guid, $this->voters)) {
            return;
        }

        $this->voters[] = $guid;
        $this->votes[$map] ++;

        $str = $this->mod->getLngString('mapvotevoted', array('<MAP>', '<GAMETYPE>'), array($this->mod->getLongMapName($this->chosen[$map]['map']), $this->mod->getLongGametype($this->chosen[$map]['gametype'])));
        $this->players[$guid]->say($str);
    }
}
