<?php
/**
 * ManuAdminMod
 *
 * This is a powerful and platform independent software for administrating game servers of various kinds.
 * If you need help with installing or using this software, please visit our website at: www.ManuAdminMod.de
 * If you want to obtain additional permissions extending the license, contact us at: Webmaster@ManuAdminMod.de
 *
 * @copyright www.ManuAdminMod.de
 * @license http://www.creativecommons.org/licenses/by-nc-nd/4.0/ Creative Commons BY-NC-ND 4.0
 * @version 1.0.0-Beta+7
 * */

namespace MAM\Plugins\Nameprotection;

use MAM\Daemon\Core\Utils;

/**
 * Nameprotection for admins
 *
 * @author ManuAdminMod
 * @todo Rename Functions with Underline
 */
class NameProtection extends Utils {
    /**
     * List of bad names
     * @var array
     */
    private $bad_names;

    /**
     * Inits the plugin
     *
     * This function initiates the plugin. This means that it register commands
     * default values, and events. It's important that every plugin has this function
     * Otherwise the plugin exists but can't be used
     */
    public function initPlugin() {
        parent::initPlugin();

        //Check Players because playerJoined isn't called at startup
        array_map(array($this, 'nameprotectionMain'), array_keys($this->players));

        $this->nameprotectionRead();

        $this->mod->setDefaultCV('nameprotection', 'enabled', false);
        $this->mod->setDefaultCV('nameprotection', 'badnamekick', false);
        $this->mod->setDefaultCV('nameprotection', 'badnamekickmode', 'tempban');
        $this->mod->setDefaultCV('nameprotection', 'badnamekickreason', 'Your name contains an illegal part: <PART>');
        $this->mod->setDefaultCV('nameprotection', 'adminnamekick', false);
        $this->mod->setDefaultCV('nameprotection', 'adminnamekickmode', 'tempban');
        $this->mod->setDefaultCV('nameprotection', 'adminnamekickreason', 'Your name is an admins name');
        $this->mod->setDefaultCV('nameprotection', 'maxnamechanges', false);
        $this->mod->setDefaultCV('nameprotection', 'maxnamechangeskickmode', 'tempban');
        $this->mod->setDefaultCV('nameprotection', 'maxnamechangeskickmode', 'You have changed your name too often');
        $this->mod->setDefaultCV('nameprotection', 'uppercasedisallow', false);
        $this->mod->setDefaultCV('nameprotection', 'uppercasekickreason', 'Please do not use only upper case letters in your nickname');
        $this->mod->setDefaultCV('nameprotection', 'uppercasekickmode', 'kick');
    }

    /**
     * Runs the plugin
     *
     * This function is called after all plugins were loaded and dependencies were checked.
     * In this function you can define things which you want to load, when all plugins are fine.
     *
     * If you know that you don't have any dependencies, you can remove this function.
     * Otherwise you should use it.
     *
     * In this function you should call functions like registerCommand, registerEvent, triggerEvent
     */
    public function runPlugin() {
        parent::runPlugin();

        $this->mod->registerEvent('playerJoined', 'nameprotectionMain', $this);
        $this->mod->registerEvent('playerNameChange', 'nameprotectionMain', $this);
        $this->mod->registerEvent('parseConfig', 'nameprotectionRead', $this);
    }

    /**
     * Reads out the badwnames file
     *
     * @return void
     */
    public function nameprotectionRead() {


        if (!$this->mod->getCV('nameprotection', 'enabled') || !$this->mod->getCV('nameprotection', 'badnamekick')) {
            return;
        }

        $file = $this->mod->getCV('nameprotection', 'disallowednames');

        $normal = array();
        $regexp = array();

        foreach ($file as $value) {
            $value = trim($value);
            if (stripos($value, 'regexp:') === 0) {
                $regexp[] = substr($value, 7);
            } else {
                $normal[] = $value;
            }
        }

        $this->bad_names = array('normal' => $normal, 'regexp' => $regexp);

        return;
    }

    /**
     * Checks if player uses a protected/forbidden name
     *
     * @param array $parameters
     * @return void
     * @todo Move Namechange to Namechange Checker
     * @todo Set upercasedisallow to false
     * @todo Move upercase to NameObserver class
     */
    public function nameprotectionMain($parameters) {

        if (!$this->mod->getCV('nameprotection', 'enabled')) {
            return;
        }

        if (is_array($parameters)) {
            $guid = $parameters[0];
        } else {
            $guid = $parameters;
        }

        $name = $this->players[$guid]->getName();


        if ($this->mod->getCV('nameprotection', 'adminnamekick') && !$this->permissions->hasPermissions($guid, 'nameprotection.punishment.adminname.ignore')) {
            if (!$this->mod->guidCanHaveName($guid, $name)) {
                $this->switchDoAction($this->mod->getCV('nameprotection', 'adminnamekickmode'), $this->mod->getCV('nameprotection', 'adminnamekickreason'), $guid);
                return;
            }
        }

        $kick = false;
        if ($this->mod->getCV('nameprotection', 'badnamekick') && $this->bad_names != false && !$this->permissions->hasPermissions($guid, 'nameprotection.punishment.badname.ignore')) {
            foreach ($this->bad_names['normal'] as $value) {
                if (stripos($name, $value) !== false) {
                    $part = $value;
                    $kick = true;
                    break;
                }
            }
            if (!$kick) {
                foreach ($this->bad_names['regexp'] as $value) {
                    if (preg_match('´' . str_replace('´', '\\xB4', $value) . '´i', $name, $subpatterns)) {
                        $part = $subpatterns[0];
                        $kick = true;
                        break;
                    }
                }
            }
            if ($kick) {
                $this->switchDoAction($this->mod->getCV('nameprotection', 'badnamekickmode'), str_replace('<PART>', $part, $this->mod->getCV('nameprotection', 'badnamekickreason')), $guid);
                return;
            }
        }

        if ($this->mod->getCV('nameprotection', 'maxnamechanges') > 0 && !$this->permissions->hasPermissions($guid, 'nameprotection.punishment.maxnamechanges.ignore')) {
            if (!isset($this->players[$guid]->nameprotection_nameChanges)) {
                $this->players[$guid]->nameprotection_nameChanges = 0;
            } else {
                $this->players[$guid]->nameprotection_nameChanges ++;
            }

            if ($this->players[$guid]->nameprotection_nameChanges > $this->mod->getCV('nameprotection', 'maxnamechanges')) {
                $this->switchDoAction($this->mod->getCV('nameprotection', 'maxnamechangeskickmode'), $this->mod->getCV('nameprotection', 'maxnamechangeskickreason'), $guid);
                return;
            }
        }

        if ($this->mod->getCV('nameprotection', 'uppercasedisallow') && !$this->permissions->hasPermissions($guid, 'nameprotection.punishment.uppercasedisallow.ignore')) {
            if (preg_match('|^[^a-z0-9]+$|', $name)) {
                $this->switchDoAction($this->mod->getCV('nameprotection', 'uppercasekickmode'), $this->mod->getCV('nameprotection', 'uppercasekickreason'), $guid);
                return;
            }
        }
    }

    /**
     * Switch trough punishments
     *
     * @param string $switch
     * @param string $do Reason
     * @param string $guid Player Guid
     */
    private function switchDoAction($switch, $do, $guid) {
        switch ($switch) {
            case 'tempban':
                $this->players[$guid]->tempBan($do);
                break;
            case 'kick':
                $this->players[$guid]->kick($do);
                break;
            case 'ban':
                $this->players[$guid]->ban($do);
                break;
            default:
                $this->logging->debug('Undefined Type for switch $switch');
                break;
        }
    }
}
